﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.IO;
using System.Text;
using System.Threading.Tasks;

using Microsoft.SqlServer.Management.Common;
using Microsoft.SqlServer.Management.Smo;
using Microsoft.SqlServer.Management.Smo.Agent;
using Microsoft.SqlServer.Management.Smo.Mail;
using Microsoft.SqlServer.Server;
using System.Data.SqlClient;

namespace SkeletonSetup
{
    class DataLayer
    {
        static string hostServer = "SG0SP0064RDB";
        
        public ArrayList GetServerList()
        {
            ArrayList serverList = new ArrayList();

            ServerConnection conn = new ServerConnection();
            conn.LoginSecure = true;
            conn.ServerInstance = hostServer;;
            Server srv = new Server(conn);

            //Query the SSISSVR.UTILITYDB.dbo.SSISServersList
            string sqlcmd = "SELECT ServerName FROM UTILITYDB.dbo.SSISServersList";
            SqlDataReader reader = srv.ConnectionContext.ExecuteReader(sqlcmd);

            while (reader.Read())
            {
                serverList.Add(reader["ServerName"]);
            }

            reader.Close();
            srv.ConnectionContext.Disconnect();
            return serverList;
        }

        public void GenerateObjectScript(string serverName)
        {
            //Generate the scripting for the server
            ServerConnection conn = new ServerConnection();
            conn.LoginSecure = true;
            conn.ServerInstance = serverName; 
            Server srv = new Server(conn);

            //Scripting options    
            ScriptingOptions scriptingOptions = new ScriptingOptions();
            scriptingOptions.IncludeIfNotExists = true;
            scriptingOptions.TargetServerVersion = SqlServerVersion.Version110;
            scriptingOptions.AppendToFile = false;
            string scriptPath = @"\\SG1DP0042FAS\INFRA_DBA\Scripts\ServerSkeleton\" + serverName + @"\";

            GenerateConfigurationScript(srv, scriptPath);

            if (srv.Information.Version.Major == 11 && srv.IsHadrEnabled)
            {
                foreach (AvailabilityGroup ag in srv.AvailabilityGroups)
                {
                    foreach (AvailabilityGroupListener agl in ag.AvailabilityGroupListeners)
                    {
                        scriptingOptions.FileName = scriptPath + "CreateAGListenerScript.sql";
                        agl.Script(scriptingOptions);
                        scriptingOptions.AppendToFile = true;
                    }

                    scriptingOptions.AppendToFile = false;
                    foreach (AvailabilityDatabase adb in ag.AvailabilityDatabases)
                    {
                        scriptingOptions.FileName = scriptPath + "CreateAGDatabaseScript.sql";
                        adb.Script(scriptingOptions);
                        scriptingOptions.AppendToFile = true;
                    }

                    scriptingOptions.AppendToFile = false;
                    foreach (AvailabilityReplica agr in ag.AvailabilityReplicas)
                    {
                        scriptingOptions.FileName = scriptPath + "CreateAGReplicaScript.sql";
                        agr.Script(scriptingOptions);
                        scriptingOptions.AppendToFile = true;
                    }

                    scriptingOptions.AppendToFile = false;
                    scriptingOptions.FileName = scriptPath + "CreateAGScript.sql";
                    ag.Script(scriptingOptions);
                }
            }

            if (srv.IsClustered)
            {
                scriptingOptions.FileName = scriptPath + "ClusterInformation.sql";
                srv.Script();                
            }
            
            //database
            scriptingOptions.AppendToFile = false;
            foreach (Database db in srv.Databases)
            {
                scriptingOptions.FileName = scriptPath + "CreateDatabaseScript.sql";
                db.Script(scriptingOptions);
                scriptingOptions.AppendToFile = true;
            }

            //backupdevices
            scriptingOptions.AppendToFile = false;
            foreach (BackupDevice bd in srv.BackupDevices)
            {
                scriptingOptions.FileName = scriptPath + "CreateBackupDeviceScript.sql";
                bd.Script(scriptingOptions);
                scriptingOptions.AppendToFile = true;
            }

            //triggers
            scriptingOptions.AppendToFile = false;
            foreach (Trigger trigger in srv.Triggers)
            {
                scriptingOptions.FileName = scriptPath + "CreateTriggerScript.sql";
                trigger.Script(scriptingOptions);
                scriptingOptions.AppendToFile = true;
            }


            //endpoints
            if (srv.Endpoints.Count > 1)
            {
                scriptingOptions.AppendToFile = false;
                foreach (Endpoint endpoint in srv.Endpoints)
                {
                    if (!endpoint.IsSystemObject)
                    {
                        scriptingOptions.FileName = scriptPath + "CreateEndpointScript.sql";
                        endpoint.Script(scriptingOptions);
                        scriptingOptions.AppendToFile = true;
                    }
                }
            }

            //jobscripts
            scriptingOptions.AppendToFile = false;
            foreach (Job job in srv.JobServer.Jobs)
            {
                scriptingOptions.FileName = scriptPath + "CreateAgentJobsScript.sql";
                job.Script(scriptingOptions);
                scriptingOptions.AppendToFile = true;
            }

            //linkserverscripts
            scriptingOptions.AppendToFile = false;
            foreach (LinkedServer linkedServer in srv.LinkedServers)
            {
                scriptingOptions.FileName = scriptPath + "CreateLinkedServerScript.sql";
                linkedServer.Script(scriptingOptions);
                scriptingOptions.AppendToFile = true;
            }

            //userlogins
            scriptingOptions.AppendToFile = false;
            foreach (Login login in srv.Logins)
            {
                scriptingOptions.FileName = scriptPath + "CreateLoginsScript.sql";
                login.Script(scriptingOptions);
                scriptingOptions.AppendToFile = true;
            }

            //server audit
            scriptingOptions.AppendToFile = false;
            foreach(Audit audit in srv.Audits)
            {
                scriptingOptions.FileName = scriptPath + "CreateServerAuditScript.sql";
                audit.Script(scriptingOptions);
                scriptingOptions.AppendToFile = true;
            }

            //server audit specifications
            scriptingOptions.AppendToFile = false;
            foreach (ServerAuditSpecification sas in srv.ServerAuditSpecifications)
            {
                scriptingOptions.FileName = scriptPath + "CreateServerAuditSpecificationsScript.sql";
                sas.Script(scriptingOptions);
                scriptingOptions.AppendToFile = true;
            }

            ////server roles;script out all members of each role except bulkadmin
            ////errors encountered with bulkadmin permission wise
            //scriptingOptions.AppendToFile = false;
            //foreach (ServerRole role in srv.Roles)
            //{
            //    if(role.Name != "bulkadmin")
            //    {
            //        scriptingOptions.FileName = scriptPath + "CreateRolesScript.sql";

            //        role.Script(scriptingOptions);
            //        scriptingOptions.AppendToFile = true;
            //    }
            //}

            //alerts
            scriptingOptions.AppendToFile = false;
            foreach (Alert alert in srv.JobServer.Alerts)
            {
                scriptingOptions.FileName = scriptPath + "CreateAlertsScript.sql";
                alert.Script(scriptingOptions);
                scriptingOptions.AppendToFile = true;
            }

            //operators
            scriptingOptions.AppendToFile = false;
            foreach (Operator operators in srv.JobServer.Operators)
            {
                scriptingOptions.FileName = scriptPath + "CreateOperatorsScript.sql";
                operators.Script(scriptingOptions);
                scriptingOptions.AppendToFile = true;
            }

            //mailprofiles
            scriptingOptions.AppendToFile = false;
            foreach (MailProfile mailProfile in srv.Mail.Profiles)
            {
                scriptingOptions.FileName = scriptPath + "CreateMailProfileScript.sql";
                mailProfile.Script(scriptingOptions);
                scriptingOptions.AppendToFile = true;
            }

            //mailaccounts
            scriptingOptions.AppendToFile = false;
            foreach (MailAccount mailAccount in srv.Mail.Accounts)
            {
                scriptingOptions.FileName = scriptPath + "CreateMailAccountScript.sql";
                mailAccount.Script(scriptingOptions);
                scriptingOptions.AppendToFile = true;
            }

            //proxy settings
            scriptingOptions.AppendToFile = false;
            foreach (ProxyAccount proxyAccount in srv.JobServer.ProxyAccounts)
            {
                scriptingOptions.FileName = scriptPath + "CreateProxyAccountsScript.sql";
                proxyAccount.Script(scriptingOptions);
                scriptingOptions.AppendToFile = true;
            }

            //credential accounts
            //scriptingOptions.AppendToFile = false;
            //if (srv.Credentials.Count > 1)
            //{

            //    foreach (Credential credential in srv.Credentials)
            //    {
            //        scriptingOptions.FileName = scriptPath + "CreateCredentialsScript.sql";
            //        scriptingOptions.AppendToFile = true;
            //    }               
            //}
        }

        public void GenerateConfigurationScript(Server srv, string scriptPath)
        {
            string fileName = scriptPath + "CreateConfigurationScript.sql";

            //write the configuration settings
            TextWriter textWriter = new StreamWriter(fileName);
            textWriter.WriteLine("EXEC sp_configure '" + srv.Configuration.AdHocDistributedQueriesEnabled.DisplayName + "', " + srv.Configuration.AdHocDistributedQueriesEnabled.ConfigValue.ToString());
            textWriter.WriteLine("EXEC sp_configure '" + srv.Configuration.Affinity64IOMask.DisplayName + "', " + srv.Configuration.Affinity64IOMask.ConfigValue.ToString());
            textWriter.WriteLine("EXEC sp_configure '" + srv.Configuration.Affinity64Mask.DisplayName + "', " + srv.Configuration.Affinity64Mask.ConfigValue.ToString());
            textWriter.WriteLine("EXEC sp_configure '" + srv.Configuration.AffinityIOMask.DisplayName + "', " + srv.Configuration.AffinityIOMask.ConfigValue.ToString());
            textWriter.WriteLine("EXEC sp_configure '" + srv.Configuration.AffinityMask.DisplayName + "', " + srv.Configuration.AffinityMask.ConfigValue.ToString());
            textWriter.WriteLine("EXEC sp_configure '" + srv.Configuration.AgentXPsEnabled.DisplayName + "', " + srv.Configuration.AgentXPsEnabled.ConfigValue.ToString());
            textWriter.WriteLine("EXEC sp_configure '" + srv.Configuration.AllowUpdates.DisplayName + "', " + srv.Configuration.AllowUpdates.ConfigValue.ToString());
            textWriter.WriteLine("EXEC sp_configure '" + srv.Configuration.BlockedProcessThreshold.DisplayName + "', " + srv.Configuration.BlockedProcessThreshold.ConfigValue.ToString());
            textWriter.WriteLine("EXEC sp_configure '" + srv.Configuration.C2AuditMode.DisplayName + "', " + srv.Configuration.C2AuditMode.ConfigValue.ToString());
            textWriter.WriteLine("EXEC sp_configure '" + srv.Configuration.CommonCriteriaComplianceEnabled.DisplayName + "', " + srv.Configuration.CommonCriteriaComplianceEnabled.ConfigValue.ToString());
            //2012; textWriter.WriteLine("EXEC sp_configure '" + srv.Configuration.ContainmentEnabled.DisplayName + "', " + srv.Configuration.ContainmentEnabled.ConfigValue.ToString());
            textWriter.WriteLine("EXEC sp_configure '" + srv.Configuration.CostThresholdForParallelism.DisplayName + "', " + srv.Configuration.CostThresholdForParallelism.ConfigValue.ToString());
            textWriter.WriteLine("EXEC sp_configure '" + srv.Configuration.CrossDBOwnershipChaining.DisplayName + "', " + srv.Configuration.CrossDBOwnershipChaining.ConfigValue.ToString());
            textWriter.WriteLine("EXEC sp_configure '" + srv.Configuration.CursorThreshold.DisplayName + "', " + srv.Configuration.CursorThreshold.ConfigValue.ToString());
            textWriter.WriteLine("EXEC sp_configure '" + srv.Configuration.DatabaseMailEnabled.DisplayName + "', " + srv.Configuration.DatabaseMailEnabled.ConfigValue.ToString());
            textWriter.WriteLine("EXEC sp_configure '" + srv.Configuration.DefaultBackupCompression.DisplayName + "', " + srv.Configuration.DefaultBackupCompression.ConfigValue.ToString());
            textWriter.WriteLine("EXEC sp_configure '" + srv.Configuration.DefaultFullTextLanguage.DisplayName + "', " + srv.Configuration.DefaultFullTextLanguage.ConfigValue.ToString());
            textWriter.WriteLine("EXEC sp_configure '" + srv.Configuration.DefaultLanguage.DisplayName + "', " + srv.Configuration.DefaultLanguage.ConfigValue.ToString());
            textWriter.WriteLine("EXEC sp_configure '" + srv.Configuration.DefaultTraceEnabled.DisplayName + "', " + srv.Configuration.DefaultTraceEnabled.ConfigValue.ToString());
            textWriter.WriteLine("EXEC sp_configure '" + srv.Configuration.DisallowResultsFromTriggers.DisplayName + "', " + srv.Configuration.DisallowResultsFromTriggers.ConfigValue.ToString());
            textWriter.WriteLine("EXEC sp_configure '" + srv.Configuration.IsSqlClrEnabled.DisplayName + "', " + srv.Configuration.IsSqlClrEnabled.ConfigValue.ToString());
            textWriter.WriteLine("EXEC sp_configure '" + srv.Configuration.MaxDegreeOfParallelism.DisplayName + "', " + srv.Configuration.MaxDegreeOfParallelism.ConfigValue.ToString());
            textWriter.WriteLine("EXEC sp_configure '" + srv.Configuration.MaxServerMemory.DisplayName + "', " + srv.Configuration.MaxServerMemory.ConfigValue.ToString());
            textWriter.WriteLine("EXEC sp_configure '" + srv.Configuration.MaxWorkerThreads.DisplayName + "', " + srv.Configuration.MaxWorkerThreads.ConfigValue.ToString());
            textWriter.WriteLine("EXEC sp_configure '" + srv.Configuration.MinMemoryPerQuery.DisplayName + "', " + srv.Configuration.MinMemoryPerQuery.ConfigValue.ToString());
            textWriter.WriteLine("EXEC sp_configure '" + srv.Configuration.MinServerMemory.DisplayName + "', " + srv.Configuration.MinServerMemory.ConfigValue.ToString());
            textWriter.WriteLine("EXEC sp_configure '" + srv.Configuration.NestedTriggers.DisplayName + "', " + srv.Configuration.NestedTriggers.ConfigValue.ToString());
            textWriter.WriteLine("EXEC sp_configure '" + srv.Configuration.NetworkPacketSize.DisplayName + "', " + srv.Configuration.NetworkPacketSize.ConfigValue.ToString());
            textWriter.WriteLine("EXEC sp_configure '" + srv.Configuration.OleAutomationProceduresEnabled.DisplayName + "', " + srv.Configuration.OleAutomationProceduresEnabled.ConfigValue.ToString());
            textWriter.WriteLine("EXEC sp_configure '" + srv.Configuration.OpenObjects.DisplayName + "', " + srv.Configuration.OpenObjects.ConfigValue.ToString());
            textWriter.WriteLine("EXEC sp_configure '" + srv.Configuration.OptimizeAdhocWorkloads.DisplayName + "', " + srv.Configuration.OptimizeAdhocWorkloads.ConfigValue.ToString());
            textWriter.WriteLine("EXEC sp_configure '" + srv.Configuration.RemoteAccess.DisplayName + "', " + srv.Configuration.RemoteAccess.ConfigValue.ToString());
            textWriter.WriteLine("EXEC sp_configure '" + srv.Configuration.RemoteDacConnectionsEnabled.DisplayName + "', " + srv.Configuration.RemoteDacConnectionsEnabled.ConfigValue.ToString());
            textWriter.WriteLine("EXEC sp_configure '" + srv.Configuration.RemoteLoginTimeout.DisplayName + "', " + srv.Configuration.RemoteLoginTimeout.ConfigValue.ToString());
            textWriter.WriteLine("EXEC sp_configure '" + srv.Configuration.RemoteProcTrans.DisplayName + "', " + srv.Configuration.RemoteProcTrans.ConfigValue.ToString());
            textWriter.WriteLine("EXEC sp_configure '" + srv.Configuration.RemoteQueryTimeout.DisplayName + "', " + srv.Configuration.RemoteQueryTimeout.ConfigValue.ToString());
            textWriter.WriteLine("EXEC sp_configure '" + srv.Configuration.ReplicationMaxTextSize.DisplayName + "', " + srv.Configuration.ReplicationMaxTextSize.ConfigValue.ToString());
            textWriter.WriteLine("EXEC sp_configure '" + srv.Configuration.ReplicationXPsEnabled.DisplayName + "', " + srv.Configuration.ReplicationXPsEnabled.ConfigValue.ToString());
            textWriter.WriteLine("EXEC sp_configure '" + srv.Configuration.ScanForStartupProcedures.DisplayName + "', " + srv.Configuration.ScanForStartupProcedures.ConfigValue.ToString());
            textWriter.WriteLine("EXEC sp_configure '" + srv.Configuration.ServerTriggerRecursionEnabled.DisplayName + "', " + srv.Configuration.ServerTriggerRecursionEnabled.ConfigValue.ToString());
            textWriter.WriteLine("EXEC sp_configure '" + srv.Configuration.SetWorkingSetSize.DisplayName + "', " + srv.Configuration.SetWorkingSetSize.ConfigValue.ToString());
            textWriter.WriteLine("EXEC sp_configure '" + srv.Configuration.ShowAdvancedOptions.DisplayName + "', " + srv.Configuration.ShowAdvancedOptions.ConfigValue.ToString());
            textWriter.WriteLine("EXEC sp_configure '" + srv.Configuration.SmoAndDmoXPsEnabled.DisplayName + "', " + srv.Configuration.SmoAndDmoXPsEnabled.ConfigValue.ToString());
            textWriter.WriteLine("EXEC sp_configure '" + srv.Configuration.TwoDigitYearCutoff.DisplayName + "', " + srv.Configuration.TwoDigitYearCutoff.ConfigValue.ToString());
            textWriter.WriteLine("EXEC sp_configure '" + srv.Configuration.UserConnections.DisplayName + "', " + srv.Configuration.UserConnections.ConfigValue.ToString());
            textWriter.WriteLine("EXEC sp_configure '" + srv.Configuration.UserOptions.DisplayName + "', " + srv.Configuration.UserOptions.ConfigValue.ToString());
            textWriter.WriteLine("EXEC sp_configure '" + srv.Configuration.XPCmdShellEnabled.DisplayName + "', " + srv.Configuration.XPCmdShellEnabled.ConfigValue.ToString());
            textWriter.WriteLine("RECONFIGURE"); 
            textWriter.Close();
        }       
    }
}
